bl_info = {
    "name": "Favorite Objects Manager",
    "author": "1P2D",
    "version": (1, 0),
    "blender": (4 , 3, 2),
    "description": "Manage favorite objects with custom shortcut and pie menu",
    "category": "Object",
}

import bpy
from bpy.types import (
    Operator,
    Menu,
    PropertyGroup,
)
from bpy.props import (
    StringProperty,
    BoolProperty,
)

# Custom property to mark objects as favorites
class FavoriteObjectProperties(PropertyGroup):
    is_favorite: BoolProperty(
        name="Is Favorite",
        default=False,
        description="Mark object as favorite"
    )

# Pie menu operator
class VIEW3D_MT_PIE_favorite_objects(Menu):
    bl_label = "Favorite Objects"

    def draw(self, context):
        layout = self.layout
        
        # Get all favorite objects
        favorite_objects = [obj for obj in bpy.data.objects if obj.favorite_object.is_favorite]
        
        # Update menu label based on favorites
        if not favorite_objects:
            self.bl_label = "Favorite Objects (empty)"
        else:
            self.bl_label = "Favorite Objects"
            
        pie = layout.menu_pie()
        for i, obj in enumerate(favorite_objects):
            pie.operator("object.select_favorite_object", text=obj.name).object_name = obj.name

# Operator to select favorite object
class SelectFavoriteObject(Operator):
    bl_idname = "object.select_favorite_object"
    bl_label = "Select Favorite Object"
    bl_options = {'REGISTER', 'UNDO'}
    
    object_name: StringProperty()
    
    def execute(self, context):
        # Deselect all objects
        bpy.ops.object.select_all(action='DESELECT')
        
        # Select and make active the favorite object
        obj = bpy.data.objects.get(self.object_name)
        if obj:
            obj.select_set(True)
            context.view_layer.objects.active = obj
        
        return {'FINISHED'}

# Toggle favorite status operator
class ToggleFavoriteObject(Operator):
    bl_idname = "object.toggle_favorite"
    bl_label = "Toggle Favorite Object"
    bl_options = {'REGISTER', 'UNDO'}
    
    @classmethod
    def description(cls, context, properties):
        obj = context.active_object
        if obj and obj.favorite_object.is_favorite:
            return "Remove selected object(s) from your favorites list"
        return "Add selected object(s) to your favorites list for quick access"
    
    def execute(self, context):
        active_obj = context.active_object
        if not active_obj:
            return {'CANCELLED'}
            
        # Get the target state based on active object (inverse of current state)
        target_state = not active_obj.favorite_object.is_favorite
        
        if target_state:  # If we're trying to add to favorites
            # Count current favorites
            current_favorites = len([obj for obj in bpy.data.objects if obj.favorite_object.is_favorite])
            # Calculate how many new objects we're trying to add
            new_objects = len([obj for obj in context.selected_objects if not obj.favorite_object.is_favorite])
            
            # Check if adding these would exceed the limit
            if current_favorites + new_objects > 8:
                self.report({'WARNING'}, "Cannot add more than 8 favorite objects (Pie menu limitation)")
                return {'CANCELLED'}
        
        # Apply to all selected objects
        modified_count = 0
        for obj in context.selected_objects:
            obj.favorite_object.is_favorite = target_state
            modified_count += 1
            
        # Add status message
        status = "removed from" if not target_state else "added to"
        if modified_count > 1:
            self.report({'INFO'}, f"{modified_count} objects {status} favorites")
        else:
            self.report({'INFO'}, f"'{active_obj.name}' {status} favorites")
            
        return {'FINISHED'}
    
    @classmethod
    def poll(cls, context):
        return context.active_object is not None

# Add to menu
def menu_func(self, context):
    obj = context.active_object
    if obj:
        is_favorite = obj.favorite_object.is_favorite
        text = "Remove from Favorite Objects" if is_favorite else "Add to Favorite Objects"
        self.layout.operator(ToggleFavoriteObject.bl_idname, text=text)

# Import the new preferences module
from . import preferences

# Keymap
addon_keymaps = []

def register():
    # Register classes
    bpy.utils.register_class(FavoriteObjectProperties)
    bpy.utils.register_class(VIEW3D_MT_PIE_favorite_objects)
    bpy.utils.register_class(SelectFavoriteObject)
    bpy.utils.register_class(ToggleFavoriteObject)
    preferences.register()
    
    # Register properties
    bpy.types.Object.favorite_object = bpy.props.PointerProperty(type=FavoriteObjectProperties)
    
    # Add to menu
    bpy.types.VIEW3D_MT_object_context_menu.append(menu_func)
    
    # Add keymap
    wm = bpy.context.window_manager
    kc = wm.keyconfigs.addon
    if kc:
        km = kc.keymaps.new(name='3D View', space_type='VIEW_3D')
        kmi = km.keymap_items.new('wm.call_menu_pie', 'LEFTMOUSE', 'PRESS', 
                                 ctrl=True, alt=True)
        kmi.properties.name = "VIEW3D_MT_PIE_favorite_objects"
        addon_keymaps.append((km, kmi))

def unregister():
    # Remove keymap
    for km, kmi in addon_keymaps:
        km.keymap_items.remove(kmi)
    addon_keymaps.clear()
    
    # Remove from menu
    bpy.types.VIEW3D_MT_object_context_menu.remove(menu_func)
    
    # Unregister classes
    preferences.unregister()
    bpy.utils.unregister_class(ToggleFavoriteObject)
    bpy.utils.unregister_class(SelectFavoriteObject)
    bpy.utils.unregister_class(VIEW3D_MT_PIE_favorite_objects)
    bpy.utils.unregister_class(FavoriteObjectProperties)

if __name__ == "__main__":
    register()